(*
 * This file is part of Barista.
 * Copyright (C) 2007-2012 Xavier Clerc.
 *
 * Barista is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Barista is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

open Utils


(* Types *)

type for_field = Name.for_class * Name.for_field * Descriptor.for_field

type for_method = Name.for_class * Name.for_method * Descriptor.for_method

type for_constructor = Name.for_class * (Descriptor.for_parameter list)

type t =
  | GetField of for_field
  | GetStatic of for_field
  | PutField of for_field
  | PutStatic of for_field
  | InvokeVirtual of for_method
  | InvokeStatic of for_method
  | InvokeSpecial of for_method
  | NewInvokeSpecial of for_constructor
  | InvokeInterface of for_method

type kind =
  | REF_getField
  | REF_getStatic
  | REF_putField
  | REF_putStatic
  | REF_invokeVirtual
  | REF_invokeStatic
  | REF_invokeSpecial
  | REF_newInvokeSpecial
  | REF_invokeInterface


(* Exception *)

BARISTA_ERROR =
  | Invalid_kind of (x : u1) ->
      Printf.sprintf "invalid reference kind (%d)" (x :> int)


(* Functions *)

let u1_of_kind x =
  let res = match x with
  | REF_getField -> 1
  | REF_getStatic -> 2
  | REF_putField -> 3
  | REF_putStatic -> 4
  | REF_invokeVirtual -> 5
  | REF_invokeStatic -> 6
  | REF_invokeSpecial -> 7
  | REF_newInvokeSpecial -> 8
  | REF_invokeInterface -> 9 in
  u1 res

let kind_of_u1 x = 
  match (x : u1 :> int) with
  | 1 -> REF_getField
  | 2 -> REF_getStatic
  | 3 -> REF_putField
  | 4 -> REF_putStatic
  | 5 -> REF_invokeVirtual
  | 6 -> REF_invokeStatic
  | 7 -> REF_invokeSpecial
  | 8 -> REF_newInvokeSpecial
  | 9 -> REF_invokeInterface
  | x -> fail (Invalid_kind (u1 x))
